#ifndef _LINUX_SWSUSP_H
#define _LINUX_SWSUSP_H

#ifdef CONFIG_SOFTWARE_SUSPEND

#if defined(SUSPEND_C) || defined(ACPI_C)
#include <asm/suspend.h>
#endif
#include <asm/atomic.h>

#define NORESUME	 1
#define RESUME_SPECIFIED 2
#define COMMANDLINE_ERROR 4

extern unsigned long swsusp_action;
extern unsigned long swsusp_result;
extern unsigned char software_suspend_state;
struct page * get_swsusp_pool_page(unsigned int gfp_mask, unsigned int order);
void free_swsusp_pool_pages(struct page *page, unsigned int order);
extern unsigned char swsusp_memory_pool_active;
extern unsigned char swsusp_freezer_state;
#define FREEZE_NEW_ACTIVITY 1
#define FREEZE_UNREFRIGERATED 2
extern atomic_t swsusp_num_active;
#define SOFTWARE_SUSPEND_IDLE 0
#define SOFTWARE_SUSPEND_DISABLED 1
#define SOFTWARE_SUSPEND_RUNNING 2
extern int swsusp_min_free;

#ifdef CONFIG_MTRR
#define SOFTWARE_SUSPEND_MTRR 1
#else
#undef SOFTWARE_SUSPEND_MTRR
#endif

#ifdef CONFIG_SOFTWARE_SUSPEND_DEBUG
extern unsigned long swsusp_debug_state;
#define PRINTFREEMEM(desn) printlog(SUSPEND_MEMORY, SUSPEND_MEDIUM, \
		"Free memory %s: %d+%d.\n", desn, nr_free_pages(), swsusp_pool_level)
#ifdef CONFIG_PREEMPT
#define PRINTPREEMPTCOUNT(desn) printlog(SUSPEND_FREEZER, SUSPEND_MEDIUM, \
		"Preempt count: %s: %d.\n", desn, current->preempt_count)
#endif
#else /* CONFIG_SOFTWARE_SUSPEND_DEBUG */
#define PRINTFREEMEM(desn) do { } while(0)
#endif /* CONFIG_SOFTWARE_SUSPEND_DEBUG */

#ifndef PRINTPREEMPTCOUNT
#define PRINTPREEMPTCOUNT(desn) do { } while(0)
#endif

/* kernel/suspend.c */
extern void software_suspend_pending(void);
extern void software_resume(void);

extern int register_suspend_notifier(struct notifier_block *);
extern int unregister_suspend_notifier(struct notifier_block *);
extern int register_resume_notifier(struct notifier_block *);
extern int unregister_resume_notifier(struct notifier_block * nb);
extern void refrigerator(unsigned long);
extern struct buffer_head * suspend_swap_bh;
extern void cleanup_finished_swsusp_io(void);
extern void __swsusp_activity_start(int, char *, char *);
extern atomic_t swsusp_num_doing_io;
#define SWSUSP_CANHOLD 1
#define SWSUSP_WAIT_ON_ME 2

#define DECLARE_SWSUSP_LOCAL_VAR \
 unsigned long swsusp_orig_value = (current->flags & PF_FRIDGE_WAIT); \
 int swsusp_local_paused = 0; \
 (void)(swsusp_orig_value); \
 (void)(swsusp_local_paused);

#define SWSUSP_THREAD_FLAGS_RESET \
 current->flags &= ~PF_FRIDGE_WAIT; \
 swsusp_orig_value = 0;

#define SWSUSP_ACTIVITY_START(flags) do { \
 if (!swsusp_orig_value) \
	__swsusp_activity_start(flags, __FUNCTION__, __FILE__); \
} while(0)

#define SWSUSP_ACTIVITY_RESTARTING(flags) do { \
 if (swsusp_local_paused) { \
	__swsusp_activity_start(flags, __FUNCTION__, __FILE__); \
	swsusp_local_paused = 0; \
 } \
} while(0)

#define SWSUSP_ACTIVITY_END do { \
 if (!swsusp_orig_value) { \
  if (current->flags & PF_FRIDGE_WAIT) { \
   current->flags &= ~PF_FRIDGE_WAIT; \
   atomic_dec(&swsusp_num_active); \
   if ((u32)atomic_read(&swsusp_num_active) > 0xffff) { \
    printk("Error: decremented swsusp_num_active below 0 in %s (%s).\n", \
     __FUNCTION__, \
     __FILE__); \
   } \
  } else \
   printk("Error Ending: %s (%d) set but doesn't have FRIDGE_WAIT now in %s (%s).\n", \
    current->comm, \
    current->pid, \
    __FUNCTION__, \
    __FILE__); \
 } \
} while(0)

#define SWSUSP_ACTIVITY_PAUSING do { \
  if ((current->flags & PF_FRIDGE_WAIT) && \
      ((swsusp_freezer_state & FREEZE_UNREFRIGERATED) || \
       (!(current->flags & PF_SYNCTHREAD)))) { \
	current->flags &= ~PF_FRIDGE_WAIT; \
	atomic_dec(&swsusp_num_active); \
	if ((u32)atomic_read(&swsusp_num_active) > 0xffff) { \
		printk("Error: decremented swsusp_num_active below 0 in %s (%s).\n", \
		__FUNCTION__, \
		__FILE__); \
	} \
  	swsusp_local_paused = 1; \
  } \
} while(0)

/* Like the above, but we decrement active count for SYNCTHREADs too.
 * Note that the thread can start new activity. Indeed, it might need
 * to when we begin our sync.
 */
#define SWSUSP_ACTIVITY_SYNCTHREAD_PAUSING do { \
  if (current->flags & PF_FRIDGE_WAIT) { \
	current->flags &= ~PF_FRIDGE_WAIT; \
	atomic_dec(&swsusp_num_active); \
	if ((u32)atomic_read(&swsusp_num_active) > 0xffff) { \
		printk("Error: decremented swsusp_num_active below 0 in %s (%s).\n", \
		__FUNCTION__, \
		__FILE__); \
	} \
  	swsusp_local_paused = 1; \
  } \
} while(0)
extern struct page * last_suspend_cache_page;
extern unsigned int suspend_task;

#define TASK_FROZEN(p) ((p->flags & PF_FROZEN) || \
			((swsusp_freezer_state & FREEZE_UNREFRIGERATED) && \
			 (!(p->flags & PF_FREEZE)) && \
			 (p->pid) && \
			 (suspend_task != p->pid)))

#else
#define software_suspend_pending()	do { } while(0)
#define software_resume()		do { } while(0)
#define register_suspend_notifier(a)	do { } while(0)
#define unregister_suspend_notifier(a)	do { } while(0)
#define refrigerator(a)			do { BUG(); } while(0)
#define DECLARE_SWSUSP_LOCAL_VAR
#define SWSUSP_ACTIVITY_START(flush)	do { } while(0)
#define SWSUSP_ACTIVITY_END		do { } while(0)
#define SWSUSP_ACTIVITY_PAUSING		do { } while(0)
#define SWSUSP_ACTIVITY_SYNCTHREAD_PAUSING do { } while(0)
#define SWSUSP_ACTIVITY_RESTARTING(flush) do { } while(0)
#define SWSUSP_THREAD_FLAGS_RESET	do { } while(0)
#define TASK_FROZEN(p) (0)
#define suspend_task (0)
#endif

#endif /* _LINUX_SWSUSP_H */
